@@ -203,6 +203,7 @@ ifapi_get_object_name(
     }
 
 cleanup:
+    context->io_state = IO_INIT;
     ifapi_cleanup_ifapi_object(&object);
     return r;
 }
@@ -271,6 +272,7 @@ ifapi_get_nv_public(
     }
 
 cleanup:
+    context->io_state = IO_INIT;
     ifapi_cleanup_ifapi_object(&object);
     return r;
 }
@@ -712,6 +714,27 @@ ifapi_sign_buffer(
     return TSS2_RC_SUCCESS;
 }
 
+static bool
+cmp_policy_ref(TPM2B_NONCE *ref1, TPM2B_NONCE *ref2)
+{
+    if ((!ref1 || !ref1->size) && (!ref2 || !ref2->size)) {
+        return true;
+    }
+    if (!ref1 || !ref1->size || !ref2 || !ref2->size)  {
+        return false;
+    }
+
+    if (ref1->size != ref2->size) {
+        return false;
+    }
+
+    if (memcmp(&ref1->buffer[0], &ref2->buffer[0], ref1->size) != 0) {
+        return false;
+    }
+
+    return true;
+}
+
 /**  Check whether public data of key is assigned to policy.
  *
  * It will be checked whether policy was authorized by abort key with public
@@ -719,29 +742,53 @@ ifapi_sign_buffer(
  *
  * @param[in] policy The policy to be checked.
  * @param[in] publicVoid The public information of the key.
- * @param[in] nameAlgVoid Not used for this compare function.
+ * @param[in] policyReferenceVoid The policy reverence to be compared.
  * @param[out] equal Switch whether check was successful.
  */
 static TSS2_RC
 equal_policy_authorization(
     TPMS_POLICY *policy,
     void *publicVoid,
-    void *nameAlgVoid,
+    void *policyRefVoid,
     bool *equal)
 {
     TPMT_PUBLIC *public = publicVoid;
-    (void)nameAlgVoid;
+    TPM2B_NONCE *policyRef = policyRefVoid;
     size_t i;
     TPML_POLICYAUTHORIZATIONS *authorizations = policy->policyAuthorizations;
+    TSS2_RC r;
+    TPM2B_PUBLIC pem_public;
 
     *equal = false;
     if (authorizations) {
         for (i = 0; i < authorizations->count; i++) {
-            if (ifapi_TPMT_PUBLIC_cmp
-                (public, &authorizations->authorizations[i].key)) {
-                *equal = true;
-                return TSS2_RC_SUCCESS;
-            }
+            if (strcmp(authorizations->authorizations[i].type, "pem") == 0) {
+                /* The public info has to be computed from the PEM key */
+                r = ifapi_get_tpm2b_public_from_pem(
+                         authorizations->authorizations[i].keyPEM, &pem_public);
+                return_if_error(r, "Invalid PEM key.");
+
+                if (pem_public.publicArea.type == TPM2_ALG_RSA) {
+                    pem_public.publicArea.parameters.rsaDetail.scheme
+                        = authorizations->authorizations[i].rsaScheme;
+                }
+                pem_public.publicArea.nameAlg = authorizations->authorizations[i].keyPEMhashAlg;
+
+                /* Check public information if key and policyRef */
+                if (ifapi_TPMT_PUBLIC_cmp(public, &pem_public.publicArea) &&
+                    cmp_policy_ref(policyRef,
+                                   &authorizations->authorizations[i].policyRef)) {
+                    *equal = true;
+                    return TSS2_RC_SUCCESS;
+                }
+            } else
+                /* Check public information if key and policyRef */
+                if (ifapi_TPMT_PUBLIC_cmp(public, &authorizations->authorizations[i].key) &&
+                    cmp_policy_ref(policyRef,
+                                   &authorizations->authorizations[i].policyRef)) {
+                    *equal = true;
+                    return TSS2_RC_SUCCESS;
+                }
         }
     }
     return TSS2_RC_SUCCESS;
@@ -1000,11 +1047,38 @@ get_policy_signature(
     TPMT_PUBLIC *public,
     TPMT_SIGNATURE *signature)
 {
+    TSS2_RC r;
     size_t i;
+    TPM2B_PUBLIC pem_public;
+    TPML_POLICYAUTHORIZATIONS *authorizations = policy->policyAuthorizations;
 
     for (i = 0; i < policy->policyAuthorizations->count; i++) {
-        if (ifapi_TPMT_PUBLIC_cmp(public,
+        if (strcmp(authorizations->authorizations[i].type, "pem") == 0) {
+            /* The public info has to be computed from the PEM key */
+            r = ifapi_get_tpm2b_public_from_pem(
+                    authorizations->authorizations[i].keyPEM, &pem_public);
+            return_if_error(r, "Invalid PEM key.");
+
+            if (pem_public.publicArea.type == TPM2_ALG_RSA) {
+                pem_public.publicArea.parameters.rsaDetail.scheme
+                    = authorizations->authorizations[i].rsaScheme;
+            }
+            pem_public.publicArea.nameAlg = authorizations->authorizations[i].keyPEMhashAlg;
+
+            if (ifapi_TPMT_PUBLIC_cmp(public, &pem_public.publicArea)) {
+                r = ifapi_der_sig_to_tpm(&pem_public.publicArea,
+                                         authorizations->authorizations[i].pemSignature.buffer,
+                                         authorizations->authorizations[i].pemSignature.size,
+                                         authorizations->authorizations[i].keyPEMhashAlg,
+                                         signature);
+
+                return_if_error(r, "Invalid signature.");
+
+                return TSS2_RC_SUCCESS;
+            }
+        } else if (ifapi_TPMT_PUBLIC_cmp(public,
                                   &policy->policyAuthorizations->authorizations[i].key)) {
+            /* The public info was already stored in the policy. */
             *signature = policy->policyAuthorizations->authorizations[i].signature;
             return TSS2_RC_SUCCESS;
         }
@@ -1075,6 +1149,7 @@ ifapi_exec_auth_policy(
     TPMT_PUBLIC *key_public,
     TPMI_ALG_HASH hash_alg,
     TPM2B_DIGEST *digest,
+    TPM2B_NONCE *policyRef,
     TPMT_SIGNATURE *signature,
     void *userdata)
 {
@@ -1113,7 +1188,7 @@ ifapi_exec_auth_policy(
         statecase(cb_ctx->cb_state, POL_CB_SEARCH_POLICY)
             r = search_policy(fapi_ctx,
                               equal_policy_authorization, true,
-                              key_public, NULL,
+                              key_public, policyRef,
                               &current_policy->policy_list);
             FAPI_SYNC(r, "Search policy", cleanup);
 
@@ -1210,7 +1285,9 @@ ifapi_exec_auth_policy(
     }
 cleanup:
     SAFE_FREE(names);
-    cleanup_policy_list(current_policy->policy_list);
+    /* Check whether cleanup was executed. */
+    if (fapi_ctx->policy.policyutil_stack)
+        cleanup_policy_list(current_policy->policy_list);
     return r;
 }
 
