@@ -30,18 +30,17 @@
 
 /*
  * This function wraps the "up-cast" of the opaque TCTI context type to the
- * type for the mssim TCTI context. The only safeguard we have to ensure this
- * operation is possible is the magic number in the mssim TCTI context.
- * If passed a NULL context, or the magic number check fails, this function
- * will return NULL.
+ * type for the mssim TCTI context. If passed a NULL context the function
+ * returns a NULL ptr. The function doesn't check magic number anymore
+ * It should checked by the appropriate tcti_common_checks.
  */
 TSS2_TCTI_MSSIM_CONTEXT*
 tcti_mssim_context_cast (TSS2_TCTI_CONTEXT *tcti_ctx)
 {
-    if (tcti_ctx != NULL && TSS2_TCTI_MAGIC (tcti_ctx) == TCTI_MSSIM_MAGIC) {
-        return (TSS2_TCTI_MSSIM_CONTEXT*)tcti_ctx;
-    }
-    return NULL;
+    if (tcti_ctx == NULL)
+        return NULL;
+
+    return (TSS2_TCTI_MSSIM_CONTEXT*)tcti_ctx;
 }
 /*
  * This function down-casts the mssim TCTI context to the common context
@@ -71,8 +70,13 @@ TSS2_RC tcti_platform_command (
     ssize_t read_ret;
 
     if (tcti_mssim == NULL) {
+        return TSS2_TCTI_RC_BAD_REFERENCE;
+    }
+
+    if (TSS2_TCTI_MAGIC (tcti_mssim) != TCTI_MSSIM_MAGIC) {
         return TSS2_TCTI_RC_BAD_CONTEXT;
     }
+
     rc = Tss2_MU_UINT32_Marshal (cmd, buf, sizeof (cmd), NULL);
     if (rc != TSS2_RC_SUCCESS) {
         LOG_ERROR ("Failed to marshal platform command %" PRIu32 ", rc: 0x%"
@@ -190,10 +194,7 @@ tcti_mssim_transmit (
     tpm_header_t header;
     TSS2_RC rc;
 
-    if (tcti_mssim == NULL) {
-        return TSS2_TCTI_RC_BAD_CONTEXT;
-    }
-    rc = tcti_common_transmit_checks (tcti_common, cmd_buf);
+    rc = tcti_common_transmit_checks (tcti_common, cmd_buf, TCTI_MSSIM_MAGIC);
     if (rc != TSS2_RC_SUCCESS) {
         return rc;
     }
@@ -231,10 +232,7 @@ tcti_mssim_cancel (
     TSS2_TCTI_COMMON_CONTEXT *tcti_common = tcti_mssim_down_cast (tcti_mssim);
     TSS2_RC rc;
 
-    if (tcti_mssim == NULL) {
-        return TSS2_TCTI_RC_BAD_CONTEXT;
-    }
-    rc = tcti_common_cancel_checks (tcti_common);
+    rc = tcti_common_cancel_checks (tcti_common, TCTI_MSSIM_MAGIC);
     if (rc != TSS2_RC_SUCCESS) {
         return rc;
     }
@@ -258,10 +256,7 @@ tcti_mssim_set_locality (
     TSS2_TCTI_COMMON_CONTEXT *tcti_common = tcti_mssim_down_cast (tcti_mssim);
     TSS2_RC rc;
 
-    if (tcti_mssim == NULL) {
-        return TSS2_TCTI_RC_BAD_CONTEXT;
-    }
-    rc = tcti_common_set_locality_checks (tcti_common);
+    rc = tcti_common_set_locality_checks (tcti_common, TCTI_MSSIM_MAGIC);
     if (rc != TSS2_RC_SUCCESS) {
         return rc;
     }
@@ -276,10 +271,27 @@ tcti_mssim_get_poll_handles (
     TSS2_TCTI_POLL_HANDLE *handles,
     size_t *num_handles)
 {
-    (void)(tctiContext);
-    (void)(handles);
-    (void)(num_handles);
-    return TSS2_TCTI_RC_NOT_IMPLEMENTED;
+    TSS2_TCTI_MSSIM_CONTEXT *tcti_mssim = tcti_mssim_context_cast (tctiContext);
+
+    if (num_handles == NULL || tcti_mssim == NULL) {
+        return TSS2_TCTI_RC_BAD_REFERENCE;
+    }
+
+    if (handles != NULL && *num_handles < 1) {
+        return TSS2_TCTI_RC_BAD_VALUE;
+    }
+
+    *num_handles = 1;
+    if (handles != NULL) {
+#ifdef _WIN32
+        *handles = tcti_mssim->tpm_sock;
+#else
+        handles->fd = tcti_mssim->tpm_sock;
+        handles->events = POLLIN | POLLOUT;
+#endif
+    }
+
+    return TSS2_RC_SUCCESS;
 }
 
 void
@@ -304,38 +316,56 @@ tcti_mssim_receive (
     unsigned char *response_buffer,
     int32_t timeout)
 {
+#ifdef TEST_FAPI_ASYNC
+    /* Used for simulating a timeout. */
+    static int wait = 0;
+#endif
     TSS2_TCTI_MSSIM_CONTEXT *tcti_mssim = tcti_mssim_context_cast (tctiContext);
     TSS2_TCTI_COMMON_CONTEXT *tcti_common = tcti_mssim_down_cast (tcti_mssim);
     TSS2_RC rc;
     UINT32 trash;
     int ret;
 
-    if (tcti_mssim == NULL) {
-        return TSS2_TCTI_RC_BAD_CONTEXT;
-    }
-    rc = tcti_common_receive_checks (tcti_common, response_size);
+    rc = tcti_common_receive_checks (tcti_common,
+                                     response_size,
+                                     TCTI_MSSIM_MAGIC);
     if (rc != TSS2_RC_SUCCESS) {
         return rc;
     }
 
     if (timeout != TSS2_TCTI_TIMEOUT_BLOCK) {
-        LOG_WARNING ("Asynchronous I/O not implemented. The 'timeout' "
-                     "parameter must be TSS2_TCTI_TIMEOUT_BLOCK.");
-        return TSS2_TCTI_RC_BAD_VALUE;
+#ifdef TEST_FAPI_ASYNC
+        if (wait < 1) {
+            LOG_TRACE("Simulating Async by requesting another invocation.");
+            wait += 1;
+            return TSS2_TCTI_RC_TRY_AGAIN;
+        } else {
+            LOG_TRACE("Sending the actual result.");
+            wait = 0;
+        }
+#endif /* TEST_FAPI_ASYNC */
     }
 
     if (tcti_common->header.size == 0) {
         /* Receive the size of the response. */
         uint8_t size_buf [sizeof (UINT32)];
-        ret = socket_recv_buf (tcti_mssim->tpm_sock, size_buf, sizeof (UINT32));
+
+        ret = socket_poll(tcti_mssim->tpm_sock, timeout);
+        if (ret != TSS2_RC_SUCCESS) {
+            if (ret == TSS2_TCTI_RC_TRY_AGAIN) {
+                return ret;
+            }
+            rc = ret;
+            goto out;
+        }
+        ret = socket_recv_buf (tcti_mssim->tpm_sock, size_buf, sizeof(UINT32));
         if (ret != sizeof (UINT32)) {
             rc = TSS2_TCTI_RC_IO_ERROR;
             goto out;
         }
 
         rc = Tss2_MU_UINT32_Unmarshal (size_buf,
-                                       sizeof (size_buf),
-                                       0,
+                                       sizeof (size_buf), 0,
                                        &tcti_common->header.size);
         if (rc != TSS2_RC_SUCCESS) {
             LOG_WARNING ("Failed to unmarshal size from tpm2 simulator "
@@ -346,18 +376,28 @@ tcti_mssim_receive (
         LOG_DEBUG ("response size: %" PRIu32, tcti_common->header.size);
     }
 
-    *response_size = tcti_common->header.size;
     if (response_buffer == NULL) {
+        *response_size = tcti_common->header.size;
         return TSS2_RC_SUCCESS;
     }
 
     if (*response_size < tcti_common->header.size) {
         *response_size = tcti_common->header.size;
+        LOG_ERROR("Response size to big: %zu > %u", *response_size, tcti_common->header.size);
         return TSS2_TCTI_RC_INSUFFICIENT_BUFFER;
     }
+    *response_size = tcti_common->header.size;
 
     /* Receive the TPM response. */
     LOG_DEBUG ("Reading response of size %" PRIu32, tcti_common->header.size);
+    ret = socket_poll(tcti_mssim->tpm_sock, timeout);
+    if (ret != TSS2_RC_SUCCESS) {
+        if (ret == TSS2_TCTI_RC_TRY_AGAIN) {
+            return ret;
+        }
+        rc = ret;
+        goto out;
+    }
     ret = socket_recv_buf (tcti_mssim->tpm_sock,
                            (unsigned char *)response_buffer,
                            tcti_common->header.size);
@@ -368,11 +408,20 @@ tcti_mssim_receive (
     LOGBLOB_DEBUG(response_buffer, tcti_common->header.size,
                   "Response buffer received:");
 
+    ret = socket_poll (tcti_mssim->tpm_sock, timeout);
+    if (ret != TSS2_RC_SUCCESS) {
+        if (ret == TSS2_TCTI_RC_TRY_AGAIN) {
+            return ret;
+        }
+        rc = ret;
+        goto out;
+    }
+
     /* Receive the appended four bytes of 0's */
     ret = socket_recv_buf (tcti_mssim->tpm_sock,
-                           (unsigned char *)&trash,
-                           4);
+                           (unsigned char *)&trash, 4);
     if (ret != 4) {
+        LOG_DEBUG ("Error reading last 4 bytes %" PRIu32, ret);
         rc = TSS2_TCTI_RC_IO_ERROR;
         goto out;
     }
@@ -559,6 +608,11 @@ Tss2_Tcti_Mssim_Init (
         goto fail_out;
     }
 
+    rc = socket_set_nonblock (tcti_mssim->tpm_sock);
+    if (rc != TSS2_RC_SUCCESS) {
+        goto fail_out;
+    }
+
     rc = socket_connect (mssim_conf.host,
                          mssim_conf.port + 1,
                          &tcti_mssim->platform_sock);
