@@ -12,19 +12,19 @@
 
 #include "tss2_esys.h"
 
-#include "test-esapi.h"
+#include "test-esys.h"
 #include "esys_iutil.h"
 #define LOGMODULE test
 #include "util/log.h"
 #include "util/aux_util.h"
 
-/** Test the ESAPI function Esys_PP_Commands.
+/** Test the ESYS function Esys_PP_Commands.
  *
  * If the test requires physical presence, the test is skipped.
  *
  *\b Note: platform authorization needed.
  *
- * Tested ESAPI commands:
+ * Tested ESYS commands:
  *  - Esys_PP_Commands() (O)
  *
  * @param[in,out] esys_context The ESYS_CONTEXT.
@@ -62,7 +62,7 @@ test_esys_pp_commands(ESYS_CONTEXT * esys_context)
         return EXIT_SUCCESS;
     }
 
-    if ((r & ~TPM2_RC_N_MASK) == TPM2_RC_BAD_AUTH) {
+    if (number_rc(r) == TPM2_RC_BAD_AUTH) {
         /* Platform authorization not possible test will be skipped */
         LOG_WARNING("Platform authorization not possible.");
         failure_return = EXIT_SKIP;
@@ -76,6 +76,6 @@ test_esys_pp_commands(ESYS_CONTEXT * esys_context)
 }
 
 int
-test_invoke_esapi(ESYS_CONTEXT * esys_context) {
+test_invoke_esys(ESYS_CONTEXT * esys_context) {
     return test_esys_pp_commands(esys_context);
 }
